#include "problem4/problem4.hpp"
#include "common/common.h"
#include "timer/clock.hpp"
#include <iostream>
#ifdef ENABLE_LOGGING
#include <log4cxx/logger.h>
#include <log4cxx/basicconfigurator.h>
#include <log4cxx/helpers/exception.h>
#include <log4cxx/patternlayout.h>
#include <log4cxx/consoleappender.h>
static const log4cxx::LogString TTCC_CONVERSION_PATTERN(LOG4CXX_STR("%-5p [%c] - %m%n"));
#endif
#include <vector>
#include <algorithm>

using namespace pe::problem4;


#ifdef ENABLE_LOGGING
/**
 * Statically initialize the log4cxx library.
 */
void configureLogging(const std::string logString, const log4cxx::LevelPtr level) {
  log4cxx::LayoutPtr layout(new log4cxx::PatternLayout(logString));
  log4cxx::AppenderPtr appender(new log4cxx::ConsoleAppender(layout));
  log4cxx::BasicConfigurator::configure(appender);
  log4cxx::LoggerPtr rootlogger = log4cxx::Logger::getRootLogger();
  rootlogger->setLevel(level);
}
#endif

int main()
{
#ifdef ENABLE_LOGGING
  std::string logPattern(TTCC_CONVERSION_PATTERN);
  log4cxx::LevelPtr logLevel = log4cxx::Level::toLevel("DEBUG");
  configureLogging(logPattern, logLevel);
#endif
  const long max=999;
  pe::Timer timer;
  Problem4 problem(4);
  std::vector<long> palindromes;
  DEFINE_LOGGER("main");

  std::cout << "Project Euler Problem 4 version ";
  std::cout <<VERSION_MAJOR<<"."<<VERSION_MINOR;
  std::cout <<"p"<<VERSION_PATCH << std::endl;

  if (problem.isPalindrome(9009)) {
    LOG_DEBUG("9009: Palindromes working");
  } else {
    LOG_DEBUG("9009: Palindromes broken");
  }

  if (problem.isPalindrome(666)) {
    LOG_DEBUG("666: Palindromes working");
  } else {
    LOG_DEBUG("666: Palindromes broken");
  }

  if (problem.isPalindrome(7863)) {
    LOG_DEBUG("7863: Palindromes broken");
  } else {
    LOG_DEBUG("7863: Palindromes working");
  }

  if (problem.isPalindrome(863)) {
    LOG_DEBUG("863: Palindromes broken");
  } else {
    LOG_DEBUG("863: Palindromes working");
  }

  timer.start();
  for( long i = 1; i <= max; i += 1) {
    for( long j = 1; j <= max; j += 1) {
      if (problem.isPalindrome(i*j)) {
        LOG_DEBUG("Found palindrome: " << i*j);
        palindromes.push_back(i*j);
      }
    }
  }
  timer.finish();

  std::cout << "Problem 4: All palindromes for max="<<max<<std::endl;
  if (! palindromes.empty()) {
    for( std::vector<long>::iterator it = palindromes.begin(); 
        it < palindromes.end(); it++) {
      std::cout << (*it) << " ";
    }
    std::cout << std::endl;
    long max_palindrome = *std::max_element(palindromes.begin(), palindromes.end());
    std::cout << "Biggest Palindrome is: "<<max_palindrome << std::endl;
  } else {
    std::cout << "No palindromes found." << std::endl;
  }
  std::cout << "Runtime was: " << timer.renderToSecond() << std::endl;

  return 0;
}
